#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <termios.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <assert.h>
#include <stdbool.h>
#include <ctype.h>

#include "dimServer.h"

#include "dis.h"

// #define DEBUG

#ifdef DEBUG
int debug = 1;
#else
int debug = 0;
#endif


#define SLEEP_TIME 0
#define BUFFSIZE 999
/*WARNING: buf size must be at least num_devices*25+3, 1503 is enough for 60 
devices*/

#define DEFAULT_PORT  "/dev/ttyS1"

char *SERIAL_DEVICE = DEFAULT_PORT;
char atmel_command = 'a';

struct termios oldsettings, newsettings;


char buf[BUFFSIZE];

int atmel_fd; //global fd for atmel


unsigned long low_thr = 0;
unsigned long high_thr = 0;


char nolink[] = "NO LINK!!!";

#define MAX_DIMSERVICES 100 /*only needed to initialize the array, but determines the upper bound of possible services*/
#define DATABUFFERSIZE 1000 /*maximal length of the data string in chars*/
#define SERVERNAMEPREFIX "mdc_gas_monitor" /*serverName is of the form "SERVERNAMEPREFIX:PID-xxx@node"*/
#define SERVICENAMESUFFIX "mdc_gas_monitor"/*SERVERNAMEPREFIX do not know if #define MAKRO1 MAKRO2 works*/

static int nofDIMServices = 0; /*the index in dimServicesArray[] of the current service*/

static struct dimService *dimServicesArray[MAX_DIMSERVICES];

static char * server_name;

int
mysleep (unsigned long int mysleep)
{
  fd_set rfds;
  struct timeval tv;
  int retval;
  unsigned long c;

  /* Watch stdin (fd 0) to see when it has input. */
  FD_ZERO (&rfds);
  FD_SET (0, &rfds);
  /* Wait up to five seconds. */
  tv.tv_sec = 0;
  tv.tv_usec = mysleep;

  retval = select (1, &rfds, NULL, NULL, &tv);
  /* Don't rely on the value of tv now! */

  for(c=0; c<=mysleep; c++) {
      retval=retval+c;   
  }

  return 0;
}


int
init_serial (char *device)
{

//open the line
  // FILE *out;
  int atmel_fd = open (device, O_RDWR | O_NOCTTY | O_NDELAY);
  if (atmel_fd == -1)
    {
      printf ("failed to open port\n");
      exit(1);
    }

  tcgetattr (atmel_fd, &oldsettings);	/*save current serial port settings */
  bzero (&newsettings, sizeof (newsettings));	/*clear the struct */
/*
  CRTSCTS : output hardware flow control (only used if the cable has
  all necessary lines. See sect. 7 of Serial-HOWTO)
  CS8     : 8n1 (8bit,no parity,1 stopbit)
  CLOCAL  : local connection, no modem contol
  CREAD   : enable receiving characters
  CBAUDEX : enable baud higher then 9600
*/

  newsettings.c_cflag = B38400 | CS8 | CREAD;
//  newsettings.c_cflag = B4800 | CS8 | CREAD | CLOCAL;
/*only standards  values for baud rate allowed. see /bits/termios.h for the 
defines and values*/

/*
  IGNPAR  : ignore bytes with parity errors
  ICRNL   : map CR to NL (otherwise a CR input on the other computer
  will not terminate input)
  otherwise make device raw (no other input processing)
*/
  newsettings.c_iflag = IGNPAR | ICRNL;

/*
  Raw output.
*/
  newsettings.c_oflag = 0;
/*
  ICANON  : enable canonical input
  disable all echo functionality, and don't send signals to calling program
*/
  newsettings.c_lflag = ICANON;
/*
  now clean the modem line and activate the settings for the port
*/
  tcflush (atmel_fd, TCIFLUSH);
  tcsetattr (atmel_fd, TCSANOW, &newsettings);
/*    terminal settings done, now handle input*/

  return atmel_fd;

}


int
input_timeout (int filedes, int seconds, int mseconds)
{
  fd_set set;
  struct timeval timeout;

  /* Initialize the file descriptor set. */
  FD_ZERO (&set);
  FD_SET (filedes, &set);

  /* Initialize the timeout data structure. */
  timeout.tv_sec = seconds;
  timeout.tv_usec = mseconds;

  /* select returns 0 if timeout, 1 if input available, -1 if error. */
  return select (FD_SETSIZE, &set, NULL, NULL, &timeout);
}


void
AtmelReceiveData (int fd)
{
  int res = 0;
//  long int messagelen;

  buf[0] = '\0';

  /*this will read from the serial port, and print a string 
     of data of res size on buf string */
  res = read (fd, (char *) buf, BUFFSIZE - 2);
  /*this will print the data on the screen */
  // printf ("read %d char\n", res);

  if (res == -1)
    {
      /*read was unsuccesfull */
      printf ("read unsuccesful\n");
      buf[0] = '\0';
    }
  else
    {
      /*read succesful, add an eof at the end of buffer, 
         needed for using printf */
      buf[res] = '\0';
    }
  /*print on the screen buf */
  printf ("read from Atmel: %s\n", buf);
  /*send the data on tcp/ip */
}


int AtmelSendCommand(int fd, char atmel_command)
{
    
    int i;
    int res = 0;
    char str[20];

    //atmel_command = 'm';		// m for measure

    switch (atmel_command)
	{
	    case 'a':
	    {
	    }
	    break;
	    case 'b':
	    {

		// tell atmel to wait for 2 theshold (8 bytes)
		res = write (fd, "b", 1);
		if (res != 1)
		{
		    printf ("unable to write on port %s\n", SERIAL_DEVICE);
		}


		sprintf (str, "%4.4ld", high_thr);
		//printf ("write to atmel: %s\n", str);
		for (i = 0; i < 4; i++)
		{
		    res = write (fd, (char *) str + i, 1);
		    if (res != 1)
		    {
			printf ("unable to write on port %s\n", SERIAL_DEVICE);
		    }
		    mysleep (10000);

		}

		sprintf (str, "%4.4ld", low_thr);
		// printf ("write to atmel: %s\n", str);
		for (i = 0; i < 4; i++)
		{
		    res = write (fd, (char *) str + i, 1);
		    if (res != 1)
		    {
			printf ("unable to write on port %s\n", SERIAL_DEVICE);
		    }
		    mysleep (10000);


		}

	    }
	    break;
	    case 'm':
	    {
		res = write (fd, "m", 1);
		if (res == -1)
		{
		    printf ("unable to write on port %s\n", SERIAL_DEVICE);
		}

	    }
	    break;
	    case 't':
	    {
		res = write (fd, "t", 1);
		if (res == -1)
		{
		    printf ("unable to write on port %s\n", SERIAL_DEVICE);
		}

	    }
	    break;
	    case 's':
	    {
		res = write (fd, "s", 1);
		if (res == -1)
		{
		    printf ("unable to write on port %s\n", SERIAL_DEVICE);
		}

	    }
	    break;
	    case '0':
	    {
		res = write (fd, "0", 1);
		if (res == -1)
		{
		    printf ("unable to write on port %s\n", SERIAL_DEVICE);
		}

	    }
	    break;
	    case 'x':
	    {
		tcsetattr (fd, TCSANOW, &oldsettings);
		printf ("server has closed the connection\n");
		return 1;

	    }
	    break;
	    default:
	    {
		printf ("unknown command: %c\n", atmel_command);

	    }
	}			//end of switch
/*
  wait 5 seconds or until data is received
*/


    int selretval = 1;

    while (selretval)
    {
	selretval = input_timeout (fd, 1, 0);
	if (selretval == -1)
	{
	    /*error happened in select function */
	    perror ("select()");
	    exit(2);
	}

	if (selretval)
	{
	    AtmelReceiveData(fd);
	}
    }

    char err_buf[45] = "No more data within 1 second.\n";
    // long int messagelen = strlen (err_buf);
    printf (err_buf);
    /* wait a short time */
    // sleep (SLEEP_TIME);
    return 0;
}





/*@michael: immediate response, because handling of asynchronity does not exist*/
void setDimSvcValue( char * svcName, char * data )
{

    if ( debug )
        printf("in setDimSvcValue, (char*) data: %s\n", data);

    if ( (NULL == svcName) || (NULL == data) )
    {
        fprintf(stderr, "In line: %i   -    - ", __LINE__ + 1);
        fprintf(stderr, "Error: void *yourInternID = %s; char data = %s;\n", svcName, data);
        return ;
    }
    /*search for the svc to be changed*/
    dimService * thisService;
    int i = 0;
    int match_flag = 0;
    for ( i = 0; i < nofDIMServices; i++ )
    {

        thisService = dimServicesArray[i];
        assert(NULL != thisService);
        if (debug) printf("In line: %i   -    - \n", __LINE__ );
        if (NULL != thisService->dimServiceName)
        {
            if (debug) printf("********************** %s \n", thisService->dimServiceName);
            //        dumpStruct(thisService);
	    

            if ( ! strcmp( thisService->dimServiceName, svcName ) )
            {
		if (debug) printf("found match: %s\n", thisService->dimServiceName);
		match_flag = 1;
                break;
            }
        }
    }

    if(!match_flag) {
	printf("******* ERROR: no match found: svcName: %s\n", svcName);
    }

    /*the new value is copied to the svc*/
    char str [1024];
    strcpy( str, data );
    strncpy(thisService->dimData, &str[0], strlen(str)+1);
    if (debug) printf("%s \n", thisService->dimServiceName);

    /*to exclude command only services*/
    if ( thisService->dimS_serviceID > 0 )
    {
        dis_update_service( thisService->dimS_serviceID );
    }
}

/*used, if e.g. client requests a write, where this method propagates the new value back to michaels side*/
void getDimSvcValue( char * svcName, char * data )
{
    /*data is your new updated value. it is a return value*/

    dimService * thisService;
    int i = 0;
    for ( i = 0; i < nofDIMServices; i++ )
    {
        thisService = dimServicesArray[i];
        if ( ! strcmp( thisService->dimServiceName, svcName ) )
        {
            break;
        }
        else
        {
            thisService = NULL;
        }
    }
    if (NULL != thisService)
    {
        char str [1024];
        strcpy( str, thisService->dimData );
        strncpy( data, &str[0], strlen( str ) + 1);
    }

    if ( debug )
        printf("Michael got the new value!\n");
}

void initDimServer()
{

    if (server_name == NULL )
    {
        /*init error and exit handler*/
        dis_add_error_handler( error_routine );
        dis_add_exit_handler ( exit_routine );

        extern int get_proc_name(char *);
        extern int get_node_name(char *);
        char proc[40];
        char node[40];
        get_proc_name(proc);
        get_node_name(node);

        int serverNameLength = strlen( SERVERNAMEPREFIX ) + strlen(":") + strlen("PID-") + strlen(proc) + strlen("@") + strlen(node) + 1;

        if ( 65536 > serverNameLength )
        {
            server_name = (char*) calloc ( serverNameLength , sizeof(char));
            if (NULL == server_name)
            {
                fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
                fprintf(stderr, "init_record: cannot allocate memory for server_name!!!");
                return ;
            }
        }
        else
        {
            fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
            fprintf(stderr, "init dimServer: string lenght of server_name exceeds 64kB !!!");
            server_name = NULL;
            return ;
        }

        sprintf(&server_name[0], "%s%s%s%s%s%s", SERVERNAMEPREFIX, ":", "PID-", proc, "@", node);
    }

}

/*names and defaults of dimService are set*/
void initStructDimService(char * service_name, char * command_name)
{
    struct dimService *thisService = NULL;
    thisService = (dimService*) calloc(1, sizeof(dimService));
    dimServicesArray[nofDIMServices] = thisService;

    /*default service*/
    clearStructDimSvc( thisService );

    char *str = NULL;
    /*analog zu str mit alloc und so*/
    char noSvcName [1024] = {'\0'};

    extern int get_node_name(char *);
    char node[40];
    get_node_name(node);
    int dns_port = dim_get_dns_port ();
    char dnsPort [1024];
    sprintf( dnsPort, "%i", dns_port );

    int serviceNameLength = strlen(SERVICENAMESUFFIX) + strlen(dnsPort) + strlen(node) + strlen(service_name) + strlen("@:-\0");

    if ( 65536 <= serviceNameLength )
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "init service_name: string lenght of service_name exceeds 64kB !!!");
        return ;
    }

    str = (char*) calloc ( serviceNameLength + 1 , sizeof(char));
    thisService->dimServiceName = (char*) calloc( serviceNameLength + 1, sizeof(char) );

    if ( (NULL == str) || (NULL == thisService->dimServiceName) )
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "init_service_name: cannot allocate memory for service_name!!!");
        return ;
    }

    if ( NULL != service_name )
    {
        thisService->serviceExists = true;
        snprintf( str, serviceNameLength + 1, "%s@%s:%s-%s", service_name, node, dnsPort, SERVICENAMESUFFIX );
        strncpy(thisService->dimServiceName, str, strlen(str));
    }
    else
    {
        thisService->serviceExists = false;
        /*if only a cmnd exists -> init noSvcName for unique ID of commandServiceName*/
        sprintf( noSvcName, "%s%i@%s:%s-%s", "noName", nofDIMServices, node, dnsPort, SERVICENAMESUFFIX );
    }
    free( str );

    if(debug) printf("svcName: %s\n", thisService->dimServiceName);
    if(debug) printf("noSvcName: %s\n", noSvcName);

    if ( command_name != NULL )
    {
        thisService->commandExists = true;

        /*command which can actually be requested, e.g. "write"*/
        str = (char*) calloc ( strlen( command_name ) + 1, sizeof(char));
        strcpy( str, command_name );
        thisService->dimCommandName = (char*) calloc(strlen(str) + 1, sizeof(char) );
        strncpy(thisService->dimCommandName, str, strlen(str));
        free( str );
        /*commandService is the unique name of the actually requestable command*/
        if ( thisService->serviceExists)
        {
            str = (char*) calloc ( strlen(command_name) + strlen(":") + strlen(thisService->dimServiceName) + 1, sizeof(char));
            sprintf( str, "%s:%s", command_name, thisService->dimServiceName );
        }
        else
        {
            str = (char*) calloc ( strlen( command_name ) + strlen(":") + strlen(noSvcName) + 1, sizeof(char));
            sprintf( str, "%s:%s", command_name, noSvcName );
        }

        if ( debug )
            printf("commandServiceName: %s\n", str );
        thisService->dimCommandServiceName = (char*) calloc(strlen(str) + 1, sizeof(char) );
        strncpy(thisService->dimCommandServiceName, str, strlen(str));
        free( str );
    }
    else
    {
        thisService->commandExists = false;
    }

    startDimService( thisService );
}

/*initial value is set and svc&cmnds are registered*/
void startDimService( struct dimService *thisService )
{


    char dimdatatype [1024];
    sprintf( dimdatatype, "C:%i", DATABUFFERSIZE );

    if ( thisService->serviceExists == true )
    {
        char data [DATABUFFERSIZE];

        /*@Michael: here you have to set the svc data for initialization*/
        /*@Michael: you have got to fill data with your new value*/
        strcpy( data, "undefined" );

        /*set the initial value of the svc*/
        thisService->dimData = (char*) calloc(DATABUFFERSIZE + 1, sizeof(char) );
        strncpy(thisService->dimData, &data[0], strlen(data));

        if (debug) dumpStruct( thisService );

        /*register DIM service */
        thisService->dimS_serviceID = dis_add_service( (thisService->dimServiceName),
                                      dimdatatype /*thisService->dimServiceFormat*/ ,
                                      thisService->dimData,
                                      (sizeof(char) * DATABUFFERSIZE) + 1 /*(strlen(thisService->dimData)+1)*/,
                                      (void *)0, 0 ); /*you could specify an user routine along with a tag, executed when the data is sent to a client.*/
        /*update??*/
    }
    else
    {
        thisService->dimS_serviceID = -1; /*default value if no service declared*/
    }

    if ( thisService->commandExists == true )
    {
        thisService->dimC_serviceID = dis_add_cmnd((thisService->dimCommandServiceName),
                                      dimdatatype /*thisService->dimCommandFormat*/ ,           /*dataType:howMany, 0 if non mixed environment*/
                                      cmndRequest,           /*serverCmndCallback,*/ /*user routine, will be executed, if the cmnd of a DIM client is received*/
                                      nofDIMServices); /*tag, identifier of command, passes to the user routine*/
    }
    else
    {
        thisService->dimC_serviceID = -1; /*default value if no cmnd occured*/
    }

    dis_start_serving( server_name );

    /*dumpStruct( thisService );*/

    nofDIMServices++;
}

/*meaning: the client wants to execute a command on this server*/
void cmndRequest(int *tag,           /*Should be long, according to the dim manual. Works with int, though.*/
                 char *data,          /*Use exactly the datatype your data consists of.
                                                                                                                                                                                           at the moment it is only int, for the threshold values.*/
                 int *size /* inclusive \0, only for dimdatatypes < limit */)
{
    /*get service*/
    dimService *service = dimServicesArray[(int) * tag];
    if ( !strcmp( service->dimCommandName, "write" ) )
    {
        if ( debug )
            printf("write request for service %s received.\n", service->dimServiceName);
        if ( debug )
            printf("Old value = %s\n", service->dimData );

        int datalength = strlen( &data[0] );
        if ( datalength < DATABUFFERSIZE )
        {
            char str [DATABUFFERSIZE];
            strcpy( str, data );
            strncpy(service->dimData, &str[0], DATABUFFERSIZE);

            /*@Michael: here you have got to propagate the new value to your side
             * e.g.:*/
            char michsNewData [1024];
            getDimSvcValue( service->dimServiceName, michsNewData );

	    if(!strcmp( service->dimServiceName, "HighThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor")) {
		high_thr = atoi( data);
		
	    }

	    if(!strcmp( service->dimServiceName, "LowThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor")) {
		low_thr = atoi( data);
	    }
	       
	    // set thresholds
	    AtmelSendCommand(atmel_fd, 'b'); 
	       

            /*@michael: synchronize with your intern value*/
            /*propagateValue( service->dimServiceName, michsNewData);*/

            dis_update_service( service->dimS_serviceID );

            if ( debug )
                printf("New value = %s\n", service->dimData );
        }
        else
        {
            fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
            fprintf(stderr, "data sent is too long. data must be < %i\n", DATABUFFERSIZE + 1 );
        }
    }
    else
    {
        if ( !strcmp( service->dimCommandName, "toggle" ) )
        {
            /*@Michael: implement what happens if toggle is received*/
            printf("Toggle received on server!\n");
        }
        else
        {
            fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
            fprintf(stderr, "unknown command, could not be executed by the server\n");
        }
    }
}

void error_routine (int severity, int error_code, char *message)
{
    fprintf(stderr, "ERROR:\n");
    fprintf(stderr, "error code: %i\n", error_code); /*look at DIM Manual for description of error codes*/
    fprintf(stderr, "severity code (0 - info, 1 - warning, 2 - error, 3 - fatal): %i\n", severity);
    fprintf(stderr, "error message: %s\n", message);
}

void exit_routine (int *code)
{
    fprintf(stderr, "Exit command was received. Error code: %i.\n", *code);
    fprintf(stderr, "But you know, I won't quit, I'll run 'til the end...\n");
}


void clearStructDimSvc( struct dimService *dimService )
{
    dimService->dimData = NULL;
    dimService->dimC_serviceID = -1;
    dimService->dimS_serviceID = -1;

    /*if svc & cmnd names should get a default value != NULL,
     * then you have to change the condition in initDIMService
     * for the setting of noName*/

    if ( NULL != dimService->dimServiceName)
    {
        free (dimService->dimServiceName);
        dimService->dimServiceName = NULL;
    }
    if ( NULL != dimService->dimCommandName)
    {
        free(dimService->dimCommandName);
        dimService->dimCommandName = NULL;
    }
    dimService->serviceExists = false;
    dimService->commandExists = false;
}

/*only for print purposes*/
void dumpStruct ( struct dimService *rec )
{
    /*	if (rec->recordAddress)
{
    printf( "%s`%s'\n","--- Record Name: ",((struct dbCommon*)rec->recordAddress)->name);
}*/
    /*    printf(rec->serverName       ? "%s`%s'\n":"%s%p\n","STRING POINTER: SERVER NAME        (serverName)       : ",rec->serverName);*/
    printf(server_name ? "%s`%s'\n" : "%s%p\n", "STRING POINTER: SERVER NAME        (serverName)       : ", server_name);
    printf(rec->dimCommandName ? "%s`%s'\n" : "%s%p\n", "STRING POINTER: COMMAND NAME       (dimCommandName)   : ", rec->dimCommandName);
    /*printf(rec->dimCommandFormat ? "%s`%s'\n":"%s%p\n","STRING POINTER: COMMAND FORMAT     (dimCommandFormat) : ",rec->dimCommandFormat);*/
    printf("FLAG          : command exists     (commandExists)    : %i (=%s)\n", rec->commandExists, rec->commandExists ? "true" : "false");
    printf("INTEGER       : Command Service ID (dimC_serviceID)   : %i\n" , rec->dimC_serviceID);
    printf(rec->dimServiceName ? "%s`%s'\n" : "%s%p\n", "STRING POINTER: SERVICE NAME       (svcName)          : ", rec->dimServiceName);
    /*printf(rec->dimServiceFormat ? "%s`%s'\n":"%s%p\n","STRING POINTER: COMMAND FORMAT     (dimServiceFormat) : ",rec->dimServiceFormat);*/
    printf("FLAG          : service exists     (serviceExists)    : %i (=%s)\n", rec->serviceExists, rec->serviceExists ? "true" : "false");
    printf("INTEGER       : Service Service ID (dimS_serviceID)   : %i\n" , rec->dimS_serviceID);
    printf("POINTER       : data Pointer       (dimData)          : %p\n" , rec->dimData);
    /*printf("CHAR          : TIER type          (dimTierType)      : '%c'\n"    ,rec->dimTierType);
      printf("POINTER       : record address     (recordAddress)    : %p\n"      ,rec->recordAddress);
      printf("INTEGER       : record index       (index)            : %i\n"      ,rec->index);
      printf(rec->recordType       ? "%s`%s'\n":"%s%p\n","STRING POINTER: record type        (recordType)       : ",rec->recordType);
      printf("INTEGER       : record type index  (recordTypeIndex)  : %i\n"      ,rec->recordTypeIndex);
      printf(rec->dimDnsAddress    ? "%s`%s'\n":"%s%p\n","STRING POINTER: DIM DNS Address    (dimDnsAddress)    : ",rec->dimDnsAddress);
      printf("INTEGER       : DIM DNS PORT       (dimDnsPort)       : %i\n"      ,rec->dimDnsPort);
      printf("INTEGER       : calledFromCallback (index)            : %i\n"      ,rec->calledFromCallback);*/
    printf("-----------------------------------------------------\n");
}




int main()
{

    // connection to Atmel
    int fd = init_serial (SERIAL_DEVICE);
    atmel_fd = fd; // global

    initDimServer();
    initStructDimService("Temperature", NULL);
    initStructDimService("HighThreshold", "write");
    initStructDimService("LowThreshold", "write");
    initStructDimService("Voltage0", NULL);
    initStructDimService("Status", NULL);
    initStructDimService("Relais1", "toggle");
    initStructDimService("Relais2", "toggle");


    /*run schema:*/
    /*    while (1)*/ /*michaelsProgIsRunning*/
    /*    {*/
    /*do michaels code*/
    /*if (value xy changed)
{
    	setDimSvcValue( xy );
    	update_service;
    	update nicht zwingend notwendig, da dic_info nur ONCE_ONLY ist. schadet aber auch nix.
}

    sleep( 1 );
      }*/

    initStructDimService("Counter", "write");
    printf("all svc initialized\n");

    /*this is dummy again for testing purposes of a dynamic service*/
    char realName [1024];
    strcpy( realName, "Counter@de-etrax042.gsi.de:2505-mdc_gas_monitor" );
    int counter = 0;
    char str [1024];
    //char data[100];

    int default_low_threshold = 80;
    low_thr = 80;
    int default_high_threshold = 100;
    high_thr = 100;


    sprintf( str, "%d", default_low_threshold );
    setDimSvcValue("LowThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor", str );

    sprintf( str, "%d", default_high_threshold );
    setDimSvcValue("HighThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor", str );

    // set thresholds

    AtmelSendCommand(atmel_fd, 'b'); 


    while (1)
    {
        //printf("in while: %i\n", counter);
        counter++;
        sprintf( str, "%i", counter );
        setDimSvcValue(realName, str );
        sleep(1);

	// get temperature
	buf[0] = '\0';
	AtmelSendCommand(fd, 't'); 
	buf[strlen(buf)-1] = '\0';
	setDimSvcValue("Temperature@de-etrax042.gsi.de:2505-mdc_gas_monitor", buf );

	// get voltage0
	AtmelSendCommand(fd, '0'); 
	buf[strlen(buf)-1] = '\0';
	setDimSvcValue("Voltage0@de-etrax042.gsi.de:2505-mdc_gas_monitor", buf );
	

	// get voltage0
	AtmelSendCommand(fd, 's'); 
	buf[strlen(buf)-1] = '\0';
	setDimSvcValue("Status@de-etrax042.gsi.de:2505-mdc_gas_monitor", buf );
	

//	getDimSvcValue( "HighThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor", data );
//	high_thr = atoi( data);

//	getDimSvcValue( "LowThreshold@de-etrax042.gsi.de:2505-mdc_gas_monitor", data );
//	low_thr = atoi( data);


	sleep(1);

    }


    tcsetattr (atmel_fd, TCSANOW, &oldsettings);
    /* wrapup, closing the serial port*/
    return 0;
}

